/* This file is part of the KDE libraries
 * SPDX-FileCopyrightText: 2009 Dario Freddi <drf at kde.org>
 *
 * SPDX-License-Identifier: LGPL-2.0-only
 */

#include "abstractsystempoller.h"

AbstractSystemPoller::AbstractSystemPoller(QObject *parent)
    : QObject(parent)
{
}

AbstractSystemPoller::~AbstractSystemPoller()
{
}

